# Copyright (C) 2016 Adam Schubert <adam.schubert@sg1-game.net>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

__author__ = "Adam Schubert <adam.schubert@sg1-game.net>"
__date__ = "$2016-01-17 14:51:02$"


import unittest
import os
import time
import locale


class TestCase(unittest.TestCase):
    def setUp(self):
        #all tests are written in en_US
        locale.setlocale(locale.LC_ALL, 'en_US.utf8')



# Copyright (C) 2016 Adam Schubert <adam.schubert@sg1-game.net>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.


def NumberToDay(dayNumber):
    return ['Sunday', 'Monday', 'Tuesday', 'Wednesday', 'Thursday', 'Friday', 'Saturday'][dayNumber]


def NumberToMonth(monthNumber):
    return ['', 'Jan', 'Feb', 'Mar', 'Apr', 'May', 'Jun', 'Jul', 'Aug', 'Sep', 'Oct', 'Nov', 'Dec'][monthNumber]



# Copyright (C) 2016 Adam Schubert <adam.schubert@sg1-game.net>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import tests.TestCase as TestCase
from cron_descriptor import Options, CasingTypeEnum, DescriptionTypeEnum, ExpressionDescriptor, MissingFieldException, FormatException, GetDescription


class TestFormats(TestCase.TestCase):

    def testEveryMinute(self):

        self.assertEqual("Every minute", GetDescription("* * * * *"))

    def testEvery1Minute(self):

        self.assertEqual("Every minute", GetDescription("*/1 * * * *"))
        self.assertEqual("Every minute", GetDescription("0 0/1 * * * ?"))

    def testEveryHour(self):

        self.assertEqual("Every hour", GetDescription("0 0 * * * ?"))
        self.assertEqual("Every hour", GetDescription("0 0 0/1 * * ?"))

    def testTimeOfDayCertainDaysOfWeek(self):

        self.assertEqual(
            "At 11:00 PM, Monday through Friday",
            GetDescription("0 23 ? * MON-FRI"))

    def testEverySecond(self):

        self.assertEqual("Every second", GetDescription("* * * * * *"))

    def testEvery45Seconds(self):

        self.assertEqual("Every 45 seconds", GetDescription("*/45 * * * * *"))

    def testEvery5Minutes(self):

        self.assertEqual("Every 05 minutes", GetDescription("*/5 * * * *"))
        self.assertEqual("Every 10 minutes", GetDescription("0 0/10 * * * ?"))

    def testEvery5MinutesOnTheSecond(self):

        self.assertEqual("Every 05 minutes", GetDescription("0 */5 * * * *"))

    def testWeekdaysAtTime(self):

        self.assertEqual(
            "At 11:30 AM, Monday through Friday",
            GetDescription("30 11 * * 1-5"))

    def testDailyAtTime(self):

        self.assertEqual("At 11:30 AM", GetDescription("30 11 * * *"))

    def testMinuteSpan(self):

        self.assertEqual(
            "Every minute between 11:00 AM and 11:10 AM", GetDescription("0-10 11 * * *"))

    def testOneMonthOnly(self):

        self.assertEqual(
            "Every minute, only in March",
            GetDescription("* * * 3 *"))

    def testTwoMonthsOnly(self):

        self.assertEqual(
            "Every minute, only in March and June",
            GetDescription("* * * 3,6 *"))

    def testTwoTimesEachAfternoon(self):

        self.assertEqual(
            "At 02:30 PM and 04:30 PM",
            GetDescription("30 14,16 * * *"))

    def testThreeTimesDaily(self):

        self.assertEqual(
            "At 06:30 AM, 02:30 PM and 04:30 PM",
            GetDescription("30 6,14,16 * * *"))

    def testOnceAWeek(self):

        self.assertEqual(
            "At 09:46 AM, only on Monday",
            GetDescription("46 9 * * 1"))

    def testDayOfMonth(self):

        self.assertEqual(
            "At 12:23 PM, on day 15 of the month",
            GetDescription("23 12 15 * *"))

    def testMonthName(self):

        self.assertEqual(
            "At 12:23 PM, only in January",
            GetDescription("23 12 * JAN *"))

    def testDayOfMonthWithQuestionMark(self):

        self.assertEqual(
            "At 12:23 PM, only in January",
            GetDescription("23 12 ? JAN *"))

    def testMonthNameRange2(self):

        self.assertEqual(
            "At 12:23 PM, January through February", GetDescription("23 12 * JAN-FEB *"))

    def testMonthNameRange3(self):

        self.assertEqual(
            "At 12:23 PM, January through March",
            GetDescription("23 12 * JAN-MAR *"))

    def testDayOfWeekName(self):

        self.assertEqual(
            "At 12:23 PM, only on Sunday",
            GetDescription("23 12 * * SUN"))

    def testDayOfWeekRange(self):

        self.assertEqual(
            "Every 05 minutes, at 03:00 PM, Monday through Friday", GetDescription("*/5 15 * * MON-FRI"))

    def testDayOfWeekOnceInMonth(self):

        self.assertEqual(
            "Every minute, on the third Monday of the month", GetDescription("* * * * MON#3"))

    def testLastDayOfTheWeekOfTheMonth(self):

        self.assertEqual(
            "Every minute, on the last Thursday of the month", GetDescription("* * * * 4L"))

    def testLastDayOfTheMonth(self):

        self.assertEqual(
            "Every 05 minutes, on the last day of the month, only in January", GetDescription("*/5 * L JAN *"))

    def testLastWeekdayOfTheMonth(self):

        self.assertEqual(
            "Every minute, on the last weekday of the month", GetDescription("* * LW * *"))

    def testLastWeekdayOfTheMonth2(self):

        self.assertEqual(
            "Every minute, on the last weekday of the month", GetDescription("* * WL * *"))

    def testFirstWeekdayOfTheMonth(self):

        self.assertEqual(
            "Every minute, on the first weekday of the month", GetDescription("* * 1W * *"))

    def testThirteenthWeekdayOfTheMonth(self):

        self.assertEqual(
            "Every minute, on the weekday nearest day 13 of the month", GetDescription("* * 13W * *"))

    def testFirstWeekdayOfTheMonth2(self):

        self.assertEqual(
            "Every minute, on the first weekday of the month", GetDescription("* * W1 * *"))

    def testParticularWeekdayOfTheMonth(self):

        self.assertEqual(
            "Every minute, on the weekday nearest day 5 of the month", GetDescription("* * 5W * *"))

    def testParticularWeekdayOfTheMonth2(self):

        self.assertEqual(
            "Every minute, on the weekday nearest day 5 of the month", GetDescription("* * W5 * *"))

    def testTimeOfDayWithSeconds(self):

        self.assertEqual("At 02:02:30 PM", GetDescription("30 02 14 * * *"))

    def testSecondInternvals(self):

        self.assertEqual(
            "Seconds 05 through 10 past the minute",
            GetDescription("5-10 * * * * *"))

    def testSecondMinutesHoursIntervals(self):

        self.assertEqual(
            "Seconds 05 through 10 past the minute, minutes 30 through 35 past the hour, between 10:00 AM and 12:59 PM", GetDescription("5-10 30-35 10-12 * * *"))

    def testEvery5MinutesAt30Seconds(self):

        self.assertEqual(
            "At 30 seconds past the minute, every 05 minutes", GetDescription("30 */5 * * * *"))

    def testMinutesPastTheHourRange(self):

        self.assertEqual(
            "At 30 minutes past the hour, between 10:00 AM and 01:59 PM, only on Wednesday and Friday", GetDescription("0 30 10-13 ? * WED,FRI"))

    def testSecondsPastTheMinuteInterval(self):

        self.assertEqual(
            "At 10 seconds past the minute, every 05 minutes", GetDescription("10 0/5 * * * ?"))

    def testBetweenWithInterval(self):

        self.assertEqual(
            "Every 03 minutes, minutes 02 through 59 past the hour, at 01:00 AM, 09:00 AM, and 10:00 PM, between day 11 and 26 of the month, January through June",
                         GetDescription("2-59/3 1,9,22 11-26 1-6 ?"))

    def testRecurringFirstOfMonth(self):

        self.assertEqual("At 06:00 AM", GetDescription("0 0 6 1/1 * ?"))

    def testMinutesPastTheHour(self):

        self.assertEqual(
            "At 05 minutes past the hour",
            GetDescription("0 5 0/1 * * ?"))

    def testOneYearOnlyWithSeconds(self):

        self.assertEqual(
            "Every second, only in 2013",
            GetDescription("* * * * * * 2013"))

    def testOneYearOnlyWithoutSeconds(self):

        self.assertEqual(
            "Every minute, only in 2013",
            GetDescription("* * * * * 2013"))

    def testTwoYearsOnly(self):

        self.assertEqual(
            "Every minute, only in 2013 and 2014", GetDescription("* * * * * 2013,2014"))

    def testYearRange2(self):

        self.assertEqual(
            "At 12:23 PM, January through February, 2013 through 2014",
                         GetDescription("23 12 * JAN-FEB * 2013-2014"))

    def testYearRange3(self):

        self.assertEqual(
            "At 12:23 PM, January through March, 2013 through 2015",
                         GetDescription("23 12 * JAN-MAR * 2013-2015"))

    def testHourRange(self):

        self.assertEqual(
            "Every 30 minutes, between 08:00 AM and 09:59 AM, on day 5 and 20 of the month", GetDescription("0 0/30 8-9 5,20 * ?"))

    def testDayOfWeekModifier(self):

        self.assertEqual(
            "At 12:23 PM, on the second Sunday of the month", GetDescription("23 12 * * SUN#2"))

    def testDayOfWeekModifierWithSundayStartOne(self):

        options = Options()
        options.DayOfWeekStartIndexZero = False

        self.assertEqual(
            "At 12:23 PM, on the second Sunday of the month", GetDescription("23 12 * * 1#2", options))

    def testDayOfWeekModifierWithDayOfMonth(self):
        self.assertEqual("At 00:00 AM, on day 1, 2, and 3 of the month, only on Wednesday and Friday", GetDescription("0 0      0 1,2,3 * WED,FRI"))

    def testHourRangeWithEveryPortion(self):

        self.assertEqual(
            "At 25 minutes past the hour, every 13 hours, between 07:00 AM and 07:59 PM", GetDescription("0 25 7-19/13 ? * *"))

    def testHourRangeWithTrailingZeroWithEveryPortion(self):

        self.assertEqual(
            "At 25 minutes past the hour, every 13 hours, between 07:00 AM and 08:59 PM", GetDescription("0 25 7-20/13 ? * *"))

    def testEvery3Day(self):

        self.assertEqual(
            "At 08:00 AM, every 3 days",
            GetDescription("0 0 8 1/3 * ? *"))

    def testsEvery3DayOfTheWeek(self):

        self.assertEqual(
            "At 10:15 AM, every 3 days of the week",
            GetDescription("0 15 10 ? * */3"))

    def testEvery3Month(self):

        self.assertEqual(
            "At 07:05 AM, on day 2 of the month, every 3 months", GetDescription("0 5 7 2 1/3 ? *"))

    def testEvery2Years(self):

        self.assertEqual(
            "At 06:15 AM, on day 1 of the month, only in January, every 2 years", GetDescription("0 15 6 1 1 ? 1/2"))

    def testMutiPartRangeSeconds(self):

        self.assertEqual(
            "Minutes 2,4 through 05 past the hour, at 01:00 AM", GetDescription("2,4-5 1 * * *"))

    def testMutiPartRangeSeconds2(self):

        self.assertEqual(
            "Minutes 2,26 through 28 past the hour, at 06:00 PM", GetDescription("2,26-28 18 * * *"))



# Copyright (C) 2016 Adam Schubert <adam.schubert@sg1-game.net>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import re
import datetime

from .l10n import l10n
from .CasingTypeEnum import CasingTypeEnum
from .DescriptionTypeEnum import DescriptionTypeEnum
from .ExpressionParser import ExpressionParser
from .Options import Options
from .Tools import NumberToDay, NumberToMonth
from .StringBuilder import StringBuilder
from .Exception import FormatException, WrongArgumentException


"""
 Converts a Cron Expression into a human readable string
"""


class ExpressionDescriptor(object):
    m_specialCharacters = ['/', '-', ',', '*']
    m_expression = ''
    m_options = None
    m_expressionParts = []
    m_parsed = False

    """
    Initializes a new instance of the ExpressionDescriptorclass
    @param: expression: The cron expression string
    @param: options: Options to control the output description
    """

    def __init__(self, expression, options=None, **kwargs):
        if options is None:
            options = Options()
        self.m_expression = expression
        self.m_options = options
        self.m_expressionParts = []
        self.m_parsed = False

        # if kwargs in m_options, overwrite it, if not raise exeption
        for kwarg in kwargs:
            if hasattr(self.m_options, kwarg):
                setattr(self.m_options, kwarg, kwargs[kwarg])
            else:
                raise WrongArgumentException(
                    "Unknow {} configuration argument".format(kwarg))

        # Initializes localizacion
        l10n()

    """
    Generates a human readable string for the Cron Expression
    @param type: Which part(s) of the expression to describe
    @returns: The cron expression description
    """

    def GetDescription(self, type=DescriptionTypeEnum.FULL):
        description = ''

        try:
            if self.m_parsed is False:
                parser = ExpressionParser(self.m_expression, self.m_options)
                self.m_expressionParts = parser.Parse()
                self.m_parsed = True

            if type == DescriptionTypeEnum.FULL:
                description = self.GetFullDescription()
            elif type == DescriptionTypeEnum.TIMEOFDAY:
                description = self.GetTimeOfDayDescription()
            elif type == DescriptionTypeEnum.HOURS:
                description = self.GetHoursDescription()
            elif type == DescriptionTypeEnum.MINUTES:
                description = self.GetMinutesDescription()
            elif type == DescriptionTypeEnum.SECONDS:
                description = self.GetSecondsDescription()
            elif type == DescriptionTypeEnum.DAYOFMONTH:
                description = self.GetDayOfMonthDescription()
            elif type == DescriptionTypeEnum.MONTH:
                description = self.GetMonthDescription()
            elif type == DescriptionTypeEnum.DAYOFWEEK:
                description = self.GetDayOfWeekDescription()
            elif type == DescriptionTypeEnum.YEAR:
                description = self.GetYearDescription()
            else:
                description = self.GetSecondsDescription()

        except Exception as ex:
            if self.m_options.ThrowExceptionOnParseError:
                raise
            else:
                description = str(ex)
        return description

    """
    Generates the FULL description
    @returns: The FULL description
    """

    def GetFullDescription(self):
        description = ''

        try:
            timeSegment = self.GetTimeOfDayDescription()
            dayOfMonthDesc = self.GetDayOfMonthDescription()
            monthDesc = self.GetMonthDescription()
            dayOfWeekDesc = self.GetDayOfWeekDescription()
            yearDesc = self.GetYearDescription()

            def dayOfWM(exp):
                if exp == "*":
                    return dayOfWeekDesc
                elif "," in exp:
                    return "{}{}".format(dayOfMonthDesc, dayOfWeekDesc)
                else:
                    return dayOfMonthDesc

            description = "{0}{1}{2}{3}".format(
                timeSegment,
                dayOfWM(self.m_expressionParts[3]),
                monthDesc,
                yearDesc)

            description = self.TransformVerbosity(
                description, self.m_options.Verbose)
            description = self.TransformCase(
                description,
                self.m_options.CasingType)
        except Exception as ex:
            description = _(
                "An error occured when generating the expression description.  Check the cron expression syntax.")
            if self.m_options.ThrowExceptionOnParseError:
                raise FormatException(description)

        return description

    """
    Generates a description for only the TIMEOFDAY portion of the expression
    @returns: The TIMEOFDAY description
    """

    def GetTimeOfDayDescription(self):
        secondsExpression = self.m_expressionParts[0]
        minuteExpression = self.m_expressionParts[1]
        hourExpression = self.m_expressionParts[2]

        description = StringBuilder()

        # handle special cases first
        if any(exp in minuteExpression for exp in self.m_specialCharacters) is False and \
            any(exp in hourExpression for exp in self.m_specialCharacters) is False and \
                any(exp in secondsExpression for exp in self.m_specialCharacters) is False:
            # specific time of day (i.e. 10 14)
            description.append(_("At "))
            description.append(
                self.FormatTime(
                    hourExpression,
                    minuteExpression,
                    secondsExpression))
        elif "-" in minuteExpression and \
            "," not in minuteExpression and \
                any(exp in hourExpression for exp in self.m_specialCharacters) is False:
            # minute range in single hour (i.e. 0-10 11)
            minuteParts = minuteExpression.split('-')
            description.append(_("Every minute between {0} and {1}").format(
                self.FormatTime(hourExpression, minuteParts[0]), self.FormatTime(hourExpression, minuteParts[1])))
        elif "," in hourExpression and any(exp in minuteExpression for exp in self.m_specialCharacters) is False:
            # hours list with single minute (o.e. 30 6,14,16)
            hourParts = hourExpression.split(',')
            description.append(_("At"))
            for i in range(0, len(hourParts)):
                description.append(" ")
                description.append(
                    self.FormatTime(hourParts[i], minuteExpression))

                if i < (len(hourParts) - 2):
                    description.append(",")

                if i == len(hourParts) - 2:
                    description.append(_(" and"))
        else:
            # default time description
            secondsDescription = self.GetSecondsDescription()
            minutesDescription = self.GetMinutesDescription()
            hoursDescription = self.GetHoursDescription()

            description.append(secondsDescription)

            if len(description) > 0:
                description.append(", ")

            description.append(minutesDescription)

            if len(description) > 0:
                description.append(", ")

            description.append(hoursDescription)
        return description.toString()

    """
    Generates a description for only the SECONDS portion of the expression
    @returns: The SECONDS description
    """

    def GetSecondsDescription(self):
        return self.GetSegmentDescription(self.m_expressionParts[0],
                                          _("every second"),
                                          lambda s: s.zfill(2),
                                          lambda s: _(
                                              "every {0} seconds").format(s),
                                          lambda s: _(
                                              "seconds {0} through {1} past the minute"),
                                          lambda s: _("at {0} seconds past the minute"))

    """
    Generates a description for only the MINUTE portion of the expression
    @returns: The MINUTE description
    """

    def GetMinutesDescription(self):
        return self.GetSegmentDescription(self.m_expressionParts[1],
                                          _("every minute"),
                                          lambda s: s.zfill(2),
                                          lambda s: _(
                                              "every {0} minutes").format(s.zfill(2)),
                                          lambda s: _(
                                              "minutes {0} through {1} past the hour"),
                                          lambda s: '' if s == "0" else _("at {0} minutes past the hour"))

    """
    Generates a description for only the HOUR portion of the expression
    @returns: The HOUR description
    """

    def GetHoursDescription(self):
        expression = self.m_expressionParts[2]
        return self.GetSegmentDescription(expression,
                                          _("every hour"),
                                          lambda s: self.FormatTime(s, "0"),
                                          lambda s: _("every {0} hours").format(
                                              s.zfill(2)),
                                          lambda s: _("between {0} and {1}"),
                                          lambda s: _("at {0}"))

    """
    Generates a description for only the DAYOFWEEK portion of the expression
    @returns: The DAYOFWEEK description
    """

    def GetDayOfWeekDescription(self):

        def GetDayName(s):
            exp = s
            if "#" in s:
                exp, useless = s.split("#", 2)
            elif "L" in s:
                exp = exp.replace("L", '')
            return NumberToDay(int(exp))

        def GetFormat(s):
            format = None
            if "#" in s:
                dayOfWeekOfMonthNumber = s[s.find("#") + 1:]
                dayOfWeekOfMonthDescription = None
                if dayOfWeekOfMonthNumber == "1":
                    dayOfWeekOfMonthDescription = _("first")
                elif dayOfWeekOfMonthNumber == "2":
                    dayOfWeekOfMonthDescription = _("second")
                elif dayOfWeekOfMonthNumber == "3":
                    dayOfWeekOfMonthDescription = _("third")
                elif dayOfWeekOfMonthNumber == "4":
                    dayOfWeekOfMonthDescription = _("forth")
                elif dayOfWeekOfMonthNumber == "5":
                    dayOfWeekOfMonthDescription = _("fifth")

                format = "{}{}{}".format(_(", on the "),
                                         dayOfWeekOfMonthDescription, _(" {0} of the month"))
            elif "L" in s:
                format = _(", on the last {0} of the month")
            else:
                format = _(", only on {0}")

            return format

        return self.GetSegmentDescription(self.m_expressionParts[5],
                                          _(", every day"),
                                          lambda s: GetDayName(s),
                                          lambda s: _(
                                              ", every {0} days of the week").format(s),
                                          lambda s: _(", {0} through {1}"),
                                          lambda s: GetFormat(s))

    """
    Generates a description for only the MONTH portion of the expression
    @returns: The MONTH description
    """

    def GetMonthDescription(self):
        return self.GetSegmentDescription(self.m_expressionParts[4],
                                          '',
                                          lambda s: datetime.date(
                                              datetime.date.today(
                                              ).year, int(
                                                  s), 1).strftime("%B"),
                                          lambda s: _(
                                              ", every {0} months").format(s),
                                          lambda s: _(", {0} through {1}"),
                                          lambda s: _(", only in {0}"))

    """
    Generates a description for only the DAYOFMONTH portion of the expression
    @returns: The DAYOFMONTH description
    """

    def GetDayOfMonthDescription(self):
        description = None
        expression = self.m_expressionParts[3]
        expression = expression.replace("?", "*")

        if expression == "L":
            description = _(", on the last day of the month")
        elif expression == "LW" or expression == "WL":
            description = _(", on the last weekday of the month")
        else:
            regex = re.compile("(\\d{1,2}W)|(W\\d{1,2})")
            if regex.match(expression):
                m = regex.match(expression)
                dayNumber = int(m.group().replace("W", ""))

                dayString = _("first weekday") if dayNumber == 1 else _("weekday nearest day {0}").format(
                    dayNumber)
                description = _(", on the {0} of the month").format(
                    dayString)
            else:
                description = self.GetSegmentDescription(expression,
                                                         _(", every day"),
                                                         lambda s: s,
                                                         lambda s: _(", every day") if s == "1" else _(
                                                             ", every {0} days"),
                                                         lambda s: _(", between day {0} and {1} of the month"),
                                                         lambda s: _(", on day {0} of the month"))

        return description

    """
    Generates a description for only the YEAR portion of the expression
    @returns: The YEAR description
    """

    def GetYearDescription(self):
        return self.GetSegmentDescription(self.m_expressionParts[6],
                                          '',
                                          lambda s: s.zfill(4),
                                          lambda s: _(", every {0} years").format(s),
                                          lambda s: _(", {0} through {1}"),
                                          lambda s: _(", only in {0}"))

    """
    Returns segment description
    @param: expression
    @param: allDescription
    @param: getSingleItemDescription
    @param: getIntervalDescriptionFormat
    @param: getBetweenDescriptionFormat
    @param: getDescriptionFormat
    @returns segment description
    """

    def GetSegmentDescription(self,
                              expression,
                              allDescription,
                              getSingleItemDescription,
                              getIntervalDescriptionFormat,
                              getBetweenDescriptionFormat,
                              getDescriptionFormat):

        description = None
        if expression is None or expression == '':
            description = ''
        elif expression == "*":
            description = allDescription
        elif any(ext in expression for ext in ['/', '-', ',']) is False:
            description = getDescriptionFormat(expression).format(
                getSingleItemDescription(expression))
        elif "/" in expression:
            segments = expression.split('/')
            description = getIntervalDescriptionFormat(
                segments[1]).format(getSingleItemDescription(segments[1]))

            # interval contains 'between' piece (i.e. 2-59/3 )
            if "-" in segments[0]:
                betweenSegmentOfInterval = segments[0]
                betweenSegements = betweenSegmentOfInterval.split('-')
                betweenSegment1Description = getSingleItemDescription(
                    betweenSegements[0])
                betweenSegment2Description = getSingleItemDescription(
                    betweenSegements[1])
                betweenSegment2Description = betweenSegment2Description.replace(
                    ":00", ":59")
                description += ", " + getBetweenDescriptionFormat(betweenSegmentOfInterval).format(
                    betweenSegment1Description, betweenSegment2Description)
        elif "-" in expression:
            segments = expression.split('-')
            betweenSegment1Description = getSingleItemDescription(segments[0])
            betweenSegment2Description = getSingleItemDescription(segments[1])
            betweenSegment2Description = betweenSegment2Description.replace(
                ":00", ":59")
            description = getBetweenDescriptionFormat(expression).format(
                betweenSegment1Description, betweenSegment2Description)
        elif "," in expression:
            segments = expression.split(',')

            descriptionContent = ''
            for i in range(0, len(segments)):
                if i > 0 and len(segments) > 2:
                    descriptionContent += ","

                    if i < len(segments) - 1:
                        descriptionContent += " "

                if i > 0 and len(segments) > 1 and (i == len(segments) - 1 or len(segments) == 2):
                    descriptionContent += _(" and ")

                descriptionContent += getSingleItemDescription(segments[i])

            description = getDescriptionFormat(
                expression).format(
                    descriptionContent)

        return description

    """
    Given time parts, will contruct a formatted time description
    @param: hourExpression Hours part
    @param: minuteExpression Minutes part
    @param: secondExpression Seconds part
    @returns: Formatted time description
    """

    def FormatTime(self,
                   hourExpression,
                   minuteExpression,
                   secondExpression=''):
        hour = int(hourExpression)

        period = ''
        if self.m_options.Use24HourTimeFormat is False:
            period = " PM" if (hour >= 12) else " AM"
            if hour > 12:
                hour -= 12

        minute = str(int(minuteExpression))  # !FIXME WUT ???
        second = ''
        if secondExpression is not None and len(secondExpression) > 0:
            second = "{}{}".format(":", str(int(secondExpression)).zfill(2))

        return "{0}:{1}{2}{3}".format(str(hour).zfill(2), minute.zfill(2), second, period)

    """
    Transforms the verbosity of the expression description by stripping verbosity from original description
    @param: description The description to transform
    @param: isVerbose If true, will leave description as it, if false, will strip verbose parts
    @returns: The transformed description with proper verbosity
    """

    def TransformVerbosity(self, description, useVerboseFormat):
        if useVerboseFormat is False:
            description = description.replace(
                _(", every minute"), '')
            description = description.replace(_(", every hour"), '')
            description = description.replace(_(", every day"), '')
        return description

    """
    Transforms the case of the expression description, based on options
    @param: description The description to transform
    @param: caseType The casing type that controls the output casing
    @returns: The transformed description with proper casing
    """

    def TransformCase(self, description, caseType):
        if caseType == CasingTypeEnum.Sentence:
            description = "{}{}".format(
                description[0].upper(),
                description[1:])
        elif caseType == CasingTypeEnum.Title:
            description = description.title()
        else:
            description = description.lower()
        return description

    def __str__(self):
        return self.GetDescription()

    def __repr__(self):
        return self.GetDescription()

"""
Generates a human readable string for the Cron Expression
@param: expression The cron expression string
@param: options Options to control the output description
@returns: The cron expression description
"""


def GetDescription(expression, options=None):
    descripter = ExpressionDescriptor(expression, options)
    return descripter.GetDescription(DescriptionTypeEnum.FULL)



# Copyright (C) 2016 Adam Schubert <adam.schubert@sg1-game.net>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import locale
import gettext
import os
import logging


class l10n(object):

    def __init__(self):
        code, encoding = locale.getlocale()
        try:
            filename = os.path.join('locale', '{}.mo'.format(code))
            trans = gettext.GNUTranslations(open(filename, "rb"))
            logging.debug('{} Loaded'.format(filename))
        except IOError:
            trans = gettext.NullTranslations()

        trans.install()



# Copyright (C) 2016 Adam Schubert <adam.schubert@sg1-game.net>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import tests.TestCase as TestCase
from cron_descriptor import Options, CasingTypeEnum, DescriptionTypeEnum, ExpressionDescriptor


class TestApi(TestCase.TestCase):

    def testFull(self):
        options = Options()
        options.Use24HourTimeFormat = True
        ceh = ExpressionDescriptor("* * * * *", options)
        self.assertEqual(
            "Every minute",
            ceh.GetDescription(DescriptionTypeEnum.FULL))

    def testDefault(self):
        self.assertEqual(
            "Every minute",
            ExpressionDescriptor("* * * * *").GetDescription())

    def testToStr(self):
        self.assertEqual(
            "Every minute",
            str(ExpressionDescriptor("* * * * *")))

    def testToRepr(self):
        self.assertIsInstance(ExpressionDescriptor("* * * * *"), ExpressionDescriptor)

    def testKargs(self):
        self.assertEqual(
            "At 17:17",
            str(ExpressionDescriptor("17 17 * * *", Use24HourTimeFormat=True)))



# Copyright (C) 2016 Adam Schubert <adam.schubert@sg1-game.net>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import re

from .Tools import NumberToDay, NumberToMonth
from .Exception import MissingFieldException, FormatException


class ExpressionParser(object):
    m_expression = ''
    m_options = None

    """
    Initializes a new instance of the <see cref="ExpressionParser"/> class
    @param: expression The cron expression string
    @param: options Parsing options
    """

    def __init__(self, expression, options):
        self.m_expression = expression
        self.m_options = options

    """
    Parses the cron expression string
    @returns: A 7 part string array, one part for each component of the cron expression (seconds, minutes, etc.)
    """

    def Parse(self):
        # Initialize all elements of parsed array to empty strings
        parsed = ['', '', '', '', '', '', '']

        if self.m_expression is None or len(self.m_expression) == 0:
            raise MissingFieldException("ExpressionDescriptor.expression")
        else:
            expressionPartsTemp = self.m_expression.split()
            expressionPartsTempLength = len(expressionPartsTemp)
            if expressionPartsTempLength < 5:
                raise FormatException(
                    "Error: Expression only has {0} parts.  At least 5 part are required.".format(
                        expressionPartsTempLength))
            elif expressionPartsTempLength == 5:
                # 5 part cron so shift array past seconds element
                parsed[1] = expressionPartsTemp[0]
                parsed[2] = expressionPartsTemp[1]
                parsed[3] = expressionPartsTemp[2]
                parsed[4] = expressionPartsTemp[3]
                parsed[5] = expressionPartsTemp[4]

            elif expressionPartsTempLength == 6:
                # If last element ends with 4 digits, a year element has been
                # supplied and no seconds element
                yearRegex = re.compile("\d{4}$")
                if yearRegex.search(expressionPartsTemp[5]) is not None:
                    parsed[1] = expressionPartsTemp[0]
                    parsed[2] = expressionPartsTemp[1]
                    parsed[3] = expressionPartsTemp[2]
                    parsed[4] = expressionPartsTemp[3]
                    parsed[5] = expressionPartsTemp[4]
                    parsed[6] = expressionPartsTemp[5]
                else:
                    for i in range(0, expressionPartsTempLength):
                        parsed[i] = expressionPartsTemp[i]
            elif expressionPartsTempLength == 7:
                parsed = expressionPartsTemp
            else:
                raise FormatException(
                    "Error: Expression has too many parts ({0}).  Expression must not have more than 7 parts.".format(
                        expressionPartsTempLength))
        self.NormalizeExpression(parsed)

        return parsed

    """
    Converts cron expression components into consistent, predictable formats.
    @param: expressionParts A 7 part string array, one part for each component of the cron expression
    """

    def NormalizeExpression(self, expressionParts):
        # convert ? to * only for DOM and DOW
        expressionParts[3] = expressionParts[3].replace("?", "*")
        expressionParts[5] = expressionParts[5].replace("?", "*")

        # convert 0/, 1/ to */
        if expressionParts[0].startswith("0/"):
            expressionParts[0] = expressionParts[
                0].replace("0/", "*/")  # seconds

        if expressionParts[1].startswith("0/"):
            expressionParts[1] = expressionParts[
                1].replace("0/", "*/")  # minutes

        if expressionParts[2].startswith("0/"):
            expressionParts[2] = expressionParts[
                2].replace("0/", "*/")  # hours

        if expressionParts[3].startswith("1/"):
            expressionParts[3] = expressionParts[3].replace("1/", "*/")  # DOM

        if expressionParts[4].startswith("1/"):
            expressionParts[4] = expressionParts[
                4].replace("1/", "*/")  # Month

        if expressionParts[5].startswith("1/"):
            expressionParts[5] = expressionParts[5].replace("1/", "*/")  # DOW

        # convert */1 to *
        length = len(expressionParts)
        for i in range(0, length):
            if expressionParts[i] == "*/1":
                expressionParts[i] = "*"

        # handle DayOfWeekStartIndexZero option where SUN=1 rather than SUN=0
        if self.m_options.DayOfWeekStartIndexZero is False:
            dowChars = list(expressionParts[5])
            for i in range(0, len(dowChars)):
                if i == 0 or dowChars[i - 1] != '#':
                    try:
                        charNumeric = int(dowChars[i])
                        dowChars[i] = str(charNumeric - 1)[0]
                    except ValueError:
                        pass
            expressionParts[5] = ''.join(dowChars)

        # convert SUN-SAT format to 0-6 format
        for i in range(0, 6):
            expressionParts[5] = expressionParts[
                5].replace(NumberToDay(i)[:3].upper(), str(i))

        # convert JAN-DEC format to 1-12 format
        for i in range(1, 13):
            expressionParts[4] = expressionParts[4].replace(
                NumberToMonth(i)[:3].upper(), str(i))

        # convert 0 second to (empty)
        if expressionParts[0] == "0":
            expressionParts[0] = ''
